import pandas as pd

# azuriranje podataka

# kreiramo recnik
studenti = {
    "ime": ["Pera", "Zika", "Mika"],
    "prezime": ["Peric", "Zikic", "Zikic"],
    "email": ["pera@peric.com", "zika@zikic.com", "mika@mikic.com"]
}

# od recnika pravimo data frame
data = pd.DataFrame(studenti)

# prikazujemo kreirani data frame
print(data)
# prikazujemo kolone u nasem skupu
print(data.columns)

# sta ako zelimo da promenimo imena kolona, na primer da ih prebacimo na engleski

# ako treba da izmenim imena svih kolona, mogu jednostavno da iskoristim dodelu
data.columns = ["First Name", "Last Name", "Email"]

# stampamo izmenjeni skup podataka
print(data)
# i izmenjene nazive kolona
print(data.columns)

# sta ako treba da promenimo imena samo odredjenih kolona
# ili samo nesto konkretno u vezi sa kolonoma? (prebaciti u mala slova,
# ili spojiti reci donjim crtama, tj. eliminisati razmake)

# prebacicemo sva imena kolona u velika slova
data.columns = [x.upper() for x in data.columns]
print(data)
# sta ako zelimo i da eliminisemo razmake, tj. da ih zamenimo sa donjim crtama
# ovo ima smisla ako koristimo pristup kolonama pomocu . notacije (npr. data.email)
# sto nije moguce ako kolone imaju razmake u imenima (npr. data.First Name)
data.columns = data.columns.str.replace(" ", "_")
print(data)

# sta ako zelimo da promenimo pojedinacne kolone, a ne sve?
# mozemo da koristimo recnik i da prosledimo one koje zelimo da promenimo
# kljuc je staro ime, a vrednost je novo ime
recnik = {
    "FIRST_NAME" : "ime",
    "LAST_NAME" : "prezime",
    "EMAIL" : "mejl"
}
# moramo da postavimo atribut inplace na True ili da izvrsimo eksplicitnu dodelu
data.rename(columns=recnik, inplace=True)
print(data)

# sta ako zelimo da izmenimo vrednosti samih podataka, tj. redove?

# sa loc i iloc mozem oda citamo i menjamo sadrzaj
red2 = data.loc[2]
print(red2)
# mozemo da koristimo i uslove za izvlacenje redova
pera = data.loc[data["ime"]=="Pera"]
print(pera)

# najjednostavniji nacin je da celom redu prosledimo nove vrednosti
red2 = ["Mika", "Mikic", "mika@mikic.com"]
# prethodni red ce samo postaviti promenljivu red2 tako da bude lista
print(red2)
# i nece imati nikakvog efekta na nas skup podataka
print(data)

# ako zelimo da promenimo skup podataka, treba direktno njemu da dodelimo
# novi sadrzaj
data.loc[2] = ["Mika", "Mikic", "mika@mikic.com"]
# i sada ce skupo podataka biti izmenjen
print(data)

# sta ako imamo veliki broj kolona, ali zelimo da promenimo samo nekoliko vrednosti

# resenje je da navedemo tacno one kolone koje zelimo da menjamo
data.loc[2, ["ime", "mejl"]] = ["Zika", "zika@mikic.com"]
# i izmenicemo samo vrednosti kolona ime i mejl u drugom redu
print(data)

# kako izmeniti samo jednu vrednost?

# tako sto je navedemo direktno, bez upotrebe listi
data.loc[2, "prezime"] = "Peric"
# i prikazemo izmenjeni skup podataka
print(data) 

# pandas ima jos jedan koristan indekser koji se naziva "at"
# i sluzi za izvlacenje ili menjanje tacno jedne vrednosti
data.at[2, "prezime"]="Zikic"
# stampamo izmenjeni skup podataka
print(data)

# cesta greska koja moze da se napravi je pokusaj da promeni
# vrednosti bez upotrebe loc ili at indeksera

# mozemo da definisemo filter kojim cemo pokupiti korisnika
# sa nekim mejlom
filterMejl = (data["mejl"] == "zika@mikic.com")
# izvlacimo podatke iz skupa podataka i menjamo 
# adresu
print(data[filterMejl]["prezime"])
# mozemo da pokusamo da promenimo prezime
# ako pazljivo pogledate rezultat komande, videce da se prijavljuje upozorenje
# koje kaze da menjamo vrednost privremenog objekta, a ne u stvarnim podacima
# (upozorenja nikada nemojte da ignorisete)
data[filterMejl]["prezime"] = "Lazic"
# ako pogledamo originalni skup podataka, videcemo da se izmena nije desila,
# jer smo izmenili vrednost kopije
print(data)

# upravo zbog ovoga treba koristiti at ili loc indeksere
data.loc[filterMejl, "prezime"] = "Lazic"
# i videcemo da je izmena napravljena direktno nad stvarnim podacima
print(data)


# sta cemo u slucaju ako treba da azuriramo vise redova odjednom?

# na primer da izmenimo sve mejl adrese tako da budu malim slovima 
# radi lakse pretrage. resenje je da izabranoj koloni dodelimo
# tu istu kolonu samo izmenjenu
data["mejl"] = data["mejl"].str.upper()
print(data)

data["mejl"] = data["mejl"].str.lower()
print(data)

# sta ako nam treba nesto malo naprednije od samo zabave sa tekstom
# postoje cetiri metoda koja se najcesce koriste
# 1. apply
# 2. map
# 3. applymap
# 4. replace

# apply se koristi da bismo pozvali neku funkciju nad nasim vrednostima, tj. podacima
# moze da se primeni na DataFrame ili na Series objekte
# ponasanje je sledece:
# Series -> primenjuje funkciju nad svakim elementom u seriji
# DataFrames -> primenjuje funkciju nad svakim redim ili koloni skupa podataka
# BITNO: apply ne menja originalni Series ili DataFrame objekat, vec vraca kopiju
#        sa rezultatom primene funkcije

# apply i primena na Series objekte

# na primer, zelimo da saznamo duzinu svih mejl adresa i prikazemo ih
# fukcija len da je duzinu stringa, pa nju treba da primenimo
# BITNO: prosledjujemo samo ime funkcije (slicno kao pokazivac na funkcije u ceo),
#        a ne njen poziv. Zbog toga, nema zagrada iza imena funkcije. 
print(data["mejl"].apply(len))


# pisemo pomocnu funkciju koja azurira mejlove (na primer, prebacuje ih u veliko slovo)
# primer je trivijalan, ali sluzi samo kao ilustracija
# funkcije pisemo kada nam treba neka komplikovanija obrada podataka
def update_email(email):
    return email.upper()
# primenjujemo funkciju i prikazujemo rezultat
print(data["mejl"].apply(update_email))

# na ovaj nacin nismo promenili vrednosti u Data Frame, samo ih izdvojili i napravili 
# njihove izmenjene kopije
# da bismo izmenili potrebno je da dodelimo nove vrednosti postojecim u skupu podataka
data["mejl"] = data["mejl"].apply(update_email)
# prikazujemo izmenjeni skup podataka
print(data)

# slicno se moze postici i lambda funkcijama, tj. anonimni funkcijama
data["mejl"] = data["mejl"].apply(lambda x: x.lower())
# prikazujemo izmenjeni skup podataka
print(data)

# drugi nacin je eksplicitna definicija lambda funkcija
brojSlova = lambda x : len(x)
print(data["mejl"].apply(brojSlova))

# apply i primena na DataFrame objekte

# ono ako primenimo len na kompletan skup podataka, kao rezultat dobicemo
# duzinu, tj. broj redova u svakoj koloni, jer funkciju primenjujemo 
# na svaku kolonu u skupu podataka, tj. na niz series objekata.
# Napomena: ovo je podrazumevano ponasanje i odgovora primeni len funkcije
#           na ukupan broj redova u svakoj koloni (tj. osa je rows)
print(data.apply(len))
# preciznije, dobijamo niz sledecih rezultat
print(len(data["ime"]), len(data["prezime"]), len(data["mejl"]))

# na isti nacin mozemo da primenimo len na svaki pojedinacni red u skupu
# podataka, treba samo da navedemo osu po kojoj zelimo da se krecemo 
# (slicno kao numpy)
# Napomena: ovde treba da primenimo len funkciju na ukupan broj kolona
#           u svakom redu (dakle, osa je columns)
print(data.apply(len,axis="columns"))

# ovde treba da koristimo funkcije koje imaju smisla da se primene nad 
# kompletnim Series objektom
# na primer, treba nam minimalna vrednost za svaku kolonu
# za stringove minimum je leksikografski prvi string
print(data.apply(pd.Series.min))
# min max i slicne statistike su mnogo korisnije za numericke podatke
# jer na njih mozemo da primenimo i do sada poznate metode iz numpy biblioteke


# mozemo da koristimo i lambde na isti nacin, samo treba da povedemo racunama
# jer lambde se ovde primenjuju nad Series
print(data.apply(lambda x: x.min()))

# postavlja se pitanje da li mozemo da primenimo funkciju na svaki pojedinacni
# element skupa podataka. i upravo za to se koristi applymap
# aplymap se koristi samo nad DataFrame objektima, ali nikad nad Series objektima

# ako prosledimo funkciju len kao argument applymap metoda, dobicemo duzine svakog
# pojedinacnog elementa u skupu podataka
print(data.applymap(len))

# ako zelimo da prebacimo sve stringove u mala slova, napisacemo
print(data.applymap(str.lower))
# treba da primenimo da je vracena izmenjena kopija originalnog skupa podataka,
#  tj. da se originalni skup podataka nije izmenio
print(data)

# map metod radi iskljucivo sa Series objektima 
# i koristi se za menjanje vrednosti svakog pojedinacnog elementa u Series objektu
# nekom drugom vrednosti koja je proizvod primene funkcije

# na primer, zelimo da zameniom neka imena
# da bismo to postigli moramo da prosledimo recnik kao argument

# kao rezultat dobicemo novi series objekat u kojem su izvrsene trazene
# izmene
# Primetite da se izmene desavaju sekvencijalno i da nakon sto Mikino ime
# promenimo u Zika, imacemo dvojicu Zika u skupu. Nakon sto se desi treca izmena,
# oba studenta koji imaju ime Zika, postace Pera
print(data["ime"].map({
    "Pera" : "Mika",
    "Mika" : "Zika",
    "Zika" : "Pera"
}))
# pored recnika, kao argument mozemo da prosledimo i funkcije/lambda funkcije koje 
# treba da se primene na svaki element Series objekta
print(data["ime"].map(lambda x: len(x)))

# problem nastaje ako ne navedemo vrednosti za sve elemente Series objekta
# one vrednost koje nisu navedene, postace NaN
print(data["ime"].map({
    "Pera" : "Mika",
    "Mika" : "Zika",
}))

# sta se desava ako ne zalim da menjam sva imena, vec samo poneko ime
# tada umesto map, treba da koristim replace. U tom slucaju, oni koje 
# nismo naveli ostaju nepromenjeni
print(data["ime"].replace({
    "Pera" : "Mika",
    "Mika" : "Zika",
}))

# imajte na umu da sve metode koje smo ilustrovali ovde (apply, applymap, map, replace)
# kao rezultat vracaju izmenjenu kopiju. da bismo tu izmenu sacuvali moramo da napravimo
# dodelu
data["ime"] = data["ime"].replace({
    "Pera" : "Mika",
    "Mika" : "Zika",
})
# i tek sa da imamo izmenjeni skup podataka
print(data)