import pandas as pd

# dodavanje/brisanje iz skupa podataka

# kreiramo recnik
studenti = {
    "ime": ["Pera", "Zika", "Mika"],
    "prezime": ["Peric", "Zikic", "Zikic"],
    "email": ["pera@peric.com", "zika@zikic.com", "mika@mikic.com"]
}

# od recnika pravimo data frame
data = pd.DataFrame(studenti)

# prikazujemo kreirani data frame
print(data)
# prikazujemo kolone u nasem skupu
print(data.columns)

# mozemo da dodamo novu kolonu podacima i to vrlo lako

# na primer hocemo da dodamo novu kolonu sa podacimo koji objedinjuju
# dve postojece kolone
data["ime_i_prezime"] = data["ime"] + " " + data["prezime"]

# stampamo izmenjeni skup podataka
print(data)

# na slican nacin mozemo da dodajemo i numericke podatke
data["godine"] = [21,22,23]
print(data)

# na njih takodje mozemo da primenimo i razne funkcije pomocu apply metode
data["godina_rodjenja"] = data["godine"].apply(lambda x : 2024 - x)
print(data)

# prilikom kreiranja kolone na ovaj nacin ne mozemo da koristimo 
# notaciju sa tackom, jer bi to znacilo da skupu podataka zelite da 
# dodelite novi objekat, a ne novu kolonu

# kao i u slucaju dodavanja kolona, mozemo i da ih uklanjamo
# za to se koristi drop metod
# drop metod samo da je pogled kako bi izgledao nas skup podataka
# nakon primene. ako zelimo da sacuvamo izmenjeni tada treba ili da 
# izvrsimo dodelu ili da postavimo atribut inplace=True
data.drop(columns=["ime", "prezime"], inplace=True)
# stampamo izmenjeni skup podataka
print(data)

# sta ako mozda zelimo da kolonu "ime_i_prezime" podelimo
# tako da dobijemo dve nove. kolonu "ime" i kolonu "prezime"

# da bismo podelili postojece vrednosti oko razmaka, mozemo da koristimo
# metod split. ako je polazna vrednost imena kolone bila "Pera Peric"
# nakon podele dobicemo listu sa dva elementa ["Pera", "Peric"]
print(data["ime_i_prezime"].str.split(" "))
# s obzirom da zelimo da napravimo dve kolone, nama ne treba jedan lista, vec
# dve liste i to cemo postici tako sto navedemo argument expand=True
print(data["ime_i_prezime"].str.split(" ", expand=True))

# prethodne naredbe su nam samo davale nove poglede, ali treba da ih dodamo 
# postojecem skupu podataka
data[["ime", "prezime"]] = data["ime_i_prezime"].str.split(" ", expand=True)
# stampamo izmenjeni skup podataka
print(data)


# kao i u slucaju kolona, mozemo da dodajemo i uklanjamo redove
# stavise mozemo i da objedinjujemo vise skupova podataka u 
# jedan novi skup

# _append metod dodaje pojedinacni red
data = data._append({"ime":"Laza"}, ignore_index=True)
# stampamo izmenjeni skup podataka
# nedostajuce informacije bice prikazane kao NaN (nedostajuce informacije)
# ignore_index=True je neophodno jer nisamo postavili index na 
# nasem skupu podataka, pa to mora pandas da uradi automatski za nas novi red
print(data)

# Bitno:
# pazljivo citajte upozorenja koja vam prikazuje pandas, jer se u njima cesto krije
# resenje problema na koji ste naisli


# pored nadovezivanja pojedinacnih redova, mozemo da nadovezujemo i kompletne
# skupove podataka

# kreiramo novi recnik 
studenti2 = {
    "ime": ["Ana", "Ena", "Ina"],
    "prezime": ["Anic", "Enic", "Inic"],
    "email": ["ana@anic.com", "ena@enic.com", "ina@inic.com"]
}

# kreiramo novi skup podatka
data2 =  pd.DataFrame(studenti2)


# mozemo da nadovezemo data2 skup podataka na data skup podataka
# Medjutim, ova dva skupa podataka imaju indekse koji se preklapaju (conflicting index)
# i imaju strukturu kolona koja se razlikuje (niti su istim redom, niti su istog tipa)

# prilikom nadovezivanja ignorisacemo indekse
# mozemo da prikazemo rezultat objedinjavanja skupova podataka
# ukoliko zelimo da sortiramo kolone prilikom objedinjavanja mozemo da prosledimo
# kao argument sort=True
print(data._append(data2, ignore_index=True))
# polazni skup podataka data nije izmenjen, vec smo samo prikazali 
# privremenu kopiju rezultata
# svi podaci koji nedostaju su obelezni kao nedostajuci, tj. NaN

# ukoliko zelimo da napravimo trajne izmene, rezultat objedinjavanja moramo
# da sacuvamo kao novi skup podataka, jer ne postoji inplace argument
objedinjeni = data._append(data2, ignore_index=True)
print(objedinjeni)

# na kraju, kao sto mozemo da dodajemo redove, mozemo i da ih
# uklanjamo
# na primer, zelimo da uklonimo studenta Ziku Zikica
# ponovo koristimo metod drop, samo sto sada ne treba da navodimo 
# imena kolona, vec indekse redova koje zelimo da uklonimo
print(data.drop(index=1))
# ako zelimo zaista da uklonimo red iz skupa podataka, treba da postavimo
# argument inplace=True
data.drop(index=1, inplace=True)
# stampamo izmenjeni skup
print(data)

# iako jednostavno, ovo nije bas fleksibilno
# mnogo fleksibilniji nacin uklanjanja redova je upotrebom 
# uslovnih izraza

# na primer, zelimo da uklonimo sve studente cije prezime Zikic

# pravimo filter
filt = objedinjeni["prezime"] == "Zikic"
# primenjujemo filter na skup podataka i izvlacimo indekse redova koji ispunjavaju 
# zadati uslov
print(objedinjeni.drop(index=objedinjeni[filt].index))