import pandas as pd
import numpy as np

# ciscenje podataka i rad sa nedostajucim vrednostima

# kreiramo recnik
studenti = {
    "ime": ["Pera", "Zika", "Mika", "Laza", np.nan, None, 'NA'],
    "prezime": ["Peric", "Zikic", "Mikic", "Lazic", np.nan, np.nan, "Missing"],
    "email": ["pera@peric.com", "zika@zikic.com", "mika@mikic.com", "laza@lazic.com", None, np.nan, "anonymus@mail.com"],
    "godine": ["22", "33", "55", "25", None, None, "Missing"]
}

# od recnika pravimo data frame
data = pd.DataFrame(studenti)

# mozemo da pogledamo kakav je nas skup podataka
print("Kompletan skup podataka: ")
print(data)

# najjednostavnije sto mozemo da uradimo sa nedostajucim podacima je da eliminisemo sve redove
# koji su nepotpuni, tj. imaju nedostajuce informacije (NaN, None)
print("Pogled na skup podataka bez nedostajucih vrednosti: ")
print(data.dropna())

# kao i sve metode do sada, dropna() ne menja direktno originalni skup podataka
# da bismo promenili originalni skup podataka, treba da navedemo argument inplace=True
data.dropna(inplace=True)
print("Skup podataka bez nedostajucih vrednosti: ")
print(data)


# podrazumevano dropna() se poziva sa sledecim argumentima
# dropna(axis="index", how="any")

# axis - definise da li eliminisemo redove ili kolone koje imaju nedostajuce vrednosti
# axis="index" - uklanja sve redove koji imaju barem jedan nedostajuci podatak
# axis="columns" - uklanja svaku kolonu koja ima barem jedan nedostajuci podatak

# how - definise kriterijum za eliminisanje redova/kolona
# how="any" - eliminise red ako ima barem jednu nedostajucu vrednost
#             medjutim, ovo cesto nije zeljeno ponasanje, jer nam mozda ne smeta ako je neki
#             podatak nedostajuci (na primer, mejl adresa)
# how="all" - eliminisace red/kolonu samo kada su sve vrednosti nedostajuce

# jasno je da ovo sto smo opisali nije dovoljno fleksibilno
# postavlja se pitanje sta cemo da radimo u slucaju kada zelimo da eliminisemo redove
# koji imaju nedostajuce vrednosti samo u nekoj konkretnoj koloni?

# na primer, sta ako nam ne smeta da nasem podatku nedostaju ime i/ili prezime, 
# ali mora imati mejl adresu
# tada iz skupa podataka treba da eliminisemo samo one redove kojima je podatak
# u koloni mejl adresa nedostajuci

# da bismo to postigli mozemo da  prosledimo argument subset
# koji predstavlja imena kolona u kojima cemo proveravati da li te
# kolone imaju nedostajuce vrednosti i samo cemo te redove eliminisati

# rekonstruisemo polazni skup podakata
data = pd.DataFrame(studenti)
# eliminisemo samo one redove kojima nedostaje mejl adresa
# s obzirom da se radi o samo jednoj koloni, argument how="any" nema neki znacaj
# jer je identican kao "all"
print(data.dropna(axis="index", how="any", subset=["email"]))
# medjutim, sta ako nam je za podatke vazno da imamo ili prezime ili mejl adresu
# tada u subset argument dodajemo jos jednu kolonu i kao rednost argumenta how
# treba da prosledimo all, jer je problem samo ako oba podatka nedostaju
print(data.dropna(axis="index", how="all", subset=["email","prezime"]))


# treba primetiti da automatski metod za eliminisanje nedostajucih vrednosti
# ne uzima u obzir specificne i korisnicke oznake da nesto nedostaje (NA, Missing i slicno)

# u zavisnosti od toga kako ucitavamo podatke, drugacijemo cemo da se pozabavimo eliminisanjem
# korisnicki definisanih nedostajucih vrednosti

# ako kreiramo podatke pomocu recnika, tada mozemo direktno u recniku da zamenimo korisnicki definisane
# vrednosti NaN i lako da resimo problem

# rekonstruisemo polazni skup podakata
data = pd.DataFrame(studenti)
# menjamo "NA" sa nan vrednosti
data.replace("NA", np.nan, inplace=True)
data.replace("Missing", np.nan, inplace=True)
# sada mozemo da pogledamo skup podataka bez korisnicki definisanih nedostajucih vrednosti
print(data)

# sada mozemo da uklonimo sve one kojima nedostaje prezime i mejl adresa
data.dropna(axis="index", how="all", subset=["email","prezime"], inplace=True)
# stampamo ocisceni skup podataka
print(data)

# ako zelimo samo da saznamo gde su nedostajuce vrednosti, ali ne i da ih uklonimo
# mozemo da koristimo isna() metod koji nam kao povratnu vrednost vraca bool matricu
print(data.isna())

# cesto kada radimo sa numerickim podacima postoji potreba da nedostajuce podatke
# popunimo nekim unapred poznatim konstantama
# na primer, ako ocenjujemo projekte studenata i neki od studenata nisu predali rad, tj.
# imamo nedostajucu vrednosti, tada ta nedostajuca vrednost zapravo znaci da je student
# osvojio nula poena
# za ovo se koristi fillna() metod
print(data.fillna("MISSING"))

# cesto postoji potreba da promenimo tip podataka
# u originalnim podacima imamo kolonu godina koja je zapravo string, a
# trebalo bi da budu celi brojevi

# da bismo proverili tipove podataka po kolonama mozemo da koristimo polje dtypes
print(data.dtypes)
# ako dobijemo da tip podataka object, tada se najcesce radi o stringu ili nekoliko
# razlicitih stvari u istom podatku

# kada kolona ima NaN vrednosti, prilikom kastovanja u broj moramo da koristimo float
# tip podataka, a to je zbog toga sto je np.nan realan broj 
data["godine"] = data["godine"].astype(float)

# i sada mozemo da vidim prosecne godine
print(data["godine"].mean())

# druga opcija je da prvo dopunimo nedostajuce vrednosti nekim stvarnim podatkom
# recimo nulom, pa da onda uradimo prebacivanje podatka u int
# cesto, ovo nije opcija, jer davanje proizvoljnih vrednosti nepoznatim podacima
# manje samu prirodu podataka

