import pandas as pd

# ciscenje podataka i rad sa nedostajucim vrednostima

# u slucaju kada ucitavamo podatke iz .csv fajla, tada mozemo direktno
# prilikom ucitavanja da eliminisemo ili zamenimo nedostajuce vrednosti
# nekim unapred definisanim konstantama

# prvo definisemo spisak mogucih korisnickih oznaka nedostajucih vrednosti
na_vals = ["NA", "Missing", "N/A"]

# ucitavamo podatke
# indeks mozemo da postavimo direktno prilikom ucitavanja podataka
# logicno je da id odgovora bude indeks

# prilikom ucitavanja fajla, mozemo odmah da definisemo koje su nedostajuce vrednosti
# i automatski ce prilikom ucitavanja sve specijalne vrednosti biti zamenjene sa NaN
data = pd.read_csv("../Data/survey_results_public.csv", index_col="ResponseId", na_values=na_vals)
data_schema = pd.read_csv("../Data/survey_results_schema.csv", index_col="qname")

# postavljamo granice
pd.set_option("display.max_columns", 114)
pd.set_option("display.max_rows", 114)

# mozemo da pokusamo da izracunamo prosecne godine iskustva programera u anketi
# postoje dve kolone koje nose te informacije
# WorkExp - radno iskustvo
# YearsCode - ukupno vreme provedeno kodirajuci uzimajuci u obzir i skolovanje i zaposlenje
print(data[["YearsCode", "WorkExp"]])

# naivno, mozemo da pokusamo da odredimo prosecne godine iskustva
# problem ce nastati, jer su podaci celobrojni i u tom slucaju ne mozemo da obradjujemo
# NaN vrednosti
# print(data[["YearsCode", "WorkExp"]].mean())

# prvo treba da kastujemo kolone u float, koji poseduje reprezentaciju NaN vrednosti
# pa tek onda da racunamo prosecnu vrednost uz pomoc mean() metoda
# mean() ce ignorisati nedostajuce vrednosti
# data["YearsCode"] = data["YearsCode"].astype(float)

# i ovo ako pokusamo da uradimo, videcemo da postoji problem, jer se u odgovorima nalaze
# odgovori poput "Less than one year" ili "More than 50 years"

# prvo treba da ispitamo koji su sve moguci odgovori u nasem skupu podataka
# pre nego sto pokusamo da uradimo kastovanje
print(data["YearsCode"].value_counts())

# ako nam samo trebaju razliciti odgovori, ali ne i statistka kojem odgovoru
# pripada koliko ispitanika mozemo da koristimo metod unique()
# kao rezultat dobijamo listu svih mogucih odgovora
print(data["YearsCode"].unique())

# zamenicemo string "Less than 1 year" sa 0
# ova zamena ima smisla, jer se radi o iskustvu koje je zaista manje od 1 godine
data["YearsCode"].replace("Less than 1 year", 0, inplace=True)
# zamenicemo string "More than 50 years" sa 51
# ova zamena ce gotovo sigurno iskriviti podatke i nije bas sjajna, jer svima koji imaju
# vise od 50 godina iskustva dodeljujemo isti broj. Medjutim, njihovo stvarno iskustvo ne moze
# biti dramaticno vece od tog broja 51, pa zamena nije bas u potpunosti neopravdana
data["YearsCode"].replace("More than 50 years", 51, inplace=True)

# provericemo da li je zamena uspela
print(data["YearsCode"].unique())

# i sada mozemo da prebacimo podatke u realne brojeve
data["YearsCode"] = data["YearsCode"].astype(float)
print("Prosecne godine iskustva: ")
print(data["YearsCode"].mean())
print("Medijalne godine iskustva: ")
print(data["YearsCode"].median())