import pandas as pd
from datetime import datetime
import matplotlib.pyplot as plt
# ucitavamo podatke o ceni bitcoin-a
# podaci su podeljeni na sate po datumima
data = pd.read_csv("../Data/timeseries.csv")

# pogledacemo strukturu podataka
print(data.head(10))

# pogledacemo i koliko podataka imamo u skupu
print(data.shape)

# kolona Date nam daje informacije o danima i satima kada je
# logovana cena bitkoina, medjutim podatak nije datetime tipa
# vec se radi o stringu, tj. object tipu podataka
print(data.dtypes)

# potrebno je da prebacimo kolonu Date u datetime podataka
# mozemo da pokusamo da prebacimo celu kolonu u datume, ali
# moramo da navedemo format u kojem su nasi podaci (pogledati u dokumentaciji)
# format="%Y-%m-%d %I-%p"
# %Y - godina zapisana sa 4 cifre
# %m - redni broj meseca zapisan sa dve cifre
# %d - redni broj dana zapisan sa dve cifre
# %I - redni broj sata zapisan sa dve cifre
# %p - obelezje dana (AM/PM)
data["Date"] = pd.to_datetime(data["Date"], format="%Y-%m-%d %I-%p")

# mozemo da proverimo da li je prebacivanje u datetime format uspelo
print(data["Date"].head(10))

# ako unapred znamo strukturu nasih podataka, vec tokom ucitavanja mozemo da prebacimo
# podatke u odgovarajuce tipove, tj. u datume u ovom slucaju
# potrebno je da navedemo imena kolona koje sadrze datume, kao i formate u kojem se nalaze
data2 = pd.read_csv("../Data/timeseries.csv", parse_dates=["Date"], date_format="%Y-%m-%d %I-%p")
print(data2.dtypes)
print(data2.head(10))

# stari nacin nacin kako ovo mozemo da postigneme jeste da koristimo lambda izraze kojim cemo
# opisati kako se datum ispravno parsira
# x ce u ovom slucaju biti pojedinacni string, tj. red koji se nalazi u Date koloni
date_parser = lambda x: datetime.strptime(x, "%Y-%m-%d %I-%p")
data3 = pd.read_csv("../Data/timeseries.csv", parse_dates=["Date"], date_parser=date_parser)
print(data3.dtypes)
print(data3.head(10))

# varijanta ucitavanja sa date_format je tekuci standard koji se koristi

# za svaki pojedinacni datum, mozemo na primer da izvucemo ime dana
print(data.loc[0,"Date"].day_name())

# medjutim, sta ako zelimo da primenimo metod day_name() na citavu kolonu, tj. na ceo series objekat?

# prvi korak je datetime svojstvima (klasi( cele serije (tj. dt polje), a zatim da pozovemo day_name() metod
# nad svim podacima u seriji
print(data["Date"].dt.day_name())

# na slican nacin pristupa i string klasi uz pomoc polja str sto nam omogucava da primenimo 
# string metode nad citavom serijom, tj. kolonom

# sta ako zelimo da kreiramo novu kolonu koja ce nam omoguciti da znamo kojeg dana se desila
# koja trgovina
data["DayOfWeek"] = data["Date"].dt.day_name()
# stampamo izmenjeni skup podataka
print(data)

# lako mozemo da odredimo neke statistike
print("Prvi dan u seriji: ", data["Date"].min())
print("Poslednji dan u seriji: ", data["Date"].max())
print("Vreme izmedju dva datuma: ", data["Date"].max() - data["Date"].min())

# na primer, zelimo da vidimo sve transakcije iz 2019
# datumi su pametni i mogu da razumeju ovakve uslove
filt = (data["Date"] >= "2019") & (data["Date"] < "2020")
# mozemo da pogledamo rezultat filtriranja
print(data.loc[filt])

# uslove mozemo da pravimo i direktnim navodjenjem datuma
filt = (data["Date"] >= pd.to_datetime("2019-01-01")) & (data["Date"] < pd.to_datetime("2020-01-01"))
# mozemo da pogledamo rezultat filtriranja
print(data.loc[filt])

# ovakav oblik filtriranja se moze svesti na odsecanje, tj. slicing
# ako prilikom ucitavanja podataka ili tokom rada postavimo index na koloni koja predstavlja datume
data.set_index("Date", inplace=True)
# provericemo da li je datum postao indeks
print(data)

# mozemo da primenimo slicing i da direktno izvucemo rezultate za 2019
print(data.loc["2019"])
# ovo nije moguce postici bez koriscenja loc indeksera
# dakle, poziv print(data["2019"]) ce prijaviti gresku

# medjutim, opseg datuma mozemo da secemo na slican nacin kao i obicne stringove
# sa razlikom da je interval zatvoren i sa leve i sa desne strane
print(data["2019-01-01":"2019-01-05"])
# ali odsecanje mozemo da radimo i po mesecima
print(data["2019-01":"2019-03"])

# na ovaj nacin lako mozemo da odredimo prosecnu cenu bitcoin-a prilikom zatvaranja berze
# za prva tri meseca 2019
print("Prosecna cena bitcoin-a za prva tri meseca 2019: ", data["2019-01":"2019-03"]["Close"].mean())

# podaci koje imamo su nam dati po satima, a sta ako zelimo da ih posmatramo po danima?
# prvo treba da odredimo sta ima smisla da gledamo na dnevnoj osnovi.

# mozemo da pogledam samo jedan dan
print(data.loc["2020-01-01"])

# smisleno je uzeti najvisu ostvarenu cenu tokom dana, sto ce biti maksimum
print(data.loc["2020-01-01"]["High"].max())
# isto tako minimalnu cenu
print(data.loc["2020-01-01"]["Low"].min())

# prakticno ono sto treba da uradimo jeste da prekodujemo podatke tako da budu po
# danima i da nam za svaki dan i podatak daju agregirane podatke
# taj postupak se naziva resampling
# trenutno su podaci u Hourly ("H") formatu. 
# ako hocemo da budu po danima (daily) kao argument treba da prosledi "D"
# na slican mozemo da prosledimo i "2D", "3D" itd
# ako nam treba na nedeljnom nivou treba da prosledimo "W" (weekly)
# (uvek pogledati u dokumentaciji pandas-a, sekcija date offsets)
# ono sto nam treba jeste da podatke prekodiramo tako da za svaki dan dobijemo max vrednost
highs = data["High"].resample("D").max()
print(highs.loc["2020-01-01"])

# mozemo da pokusamo da nacrtamo maksimalne ostvarene cene po danima
# da bi to bilo moguce moramo da imamo instaliran matplotlib
plt.plot(highs)
plt.show()


# postavlja se pitanje kako da resemplujemo vise kolona odjednom?
# mozemo da primenimo resample na citav skup podataka, tj. na vise
# kolona sa razlicitim agregatnim funkcijama
# za to koristimo metod agg() koji kao recnik prihvata recnik
# u kojima su kljucevi nazivi kolona, a vrednosti nazivi agregatnih funkcija
# koje zelimo da primenimo
daily_data = data.resample("D").agg(
{
    "Close" : "mean", 
    "High" : "max", 
    "Low" : "min", 
    "Volume" : "sum"
})
print(daily_data)
